use crate::base::i_service::IService;
use crate::utils::errors::AppError;
use async_trait::async_trait;
use sea_orm::{
    ActiveModelTrait, ColumnTrait, DatabaseConnection, EntityTrait, ModelTrait, Order, QueryFilter,
    QueryOrder, Set,
};

pub struct BaseService<Entity, Model, ActiveModel>
where
    Entity: EntityTrait,
    Model: ModelTrait + Send + Sync + 'static,
    ActiveModel: ActiveModelTrait + Send + Sync + 'static,
{
    entity: std::marker::PhantomData<Entity>,
}

impl<Entity, Model, ActiveModel> BaseService<Entity, Model, ActiveModel>
where
    Entity: EntityTrait,
    Model: ModelTrait + Send + Sync + 'static,
    ActiveModel: ActiveModelTrait + Send + Sync + 'static,
{
    pub fn new() -> Self {
        Self {
            entity: std::marker::PhantomData,
        }
    }
}

#[async_trait]
impl<Entity, Model, ActiveModel> IService<Entity, Model, ActiveModel>
    for BaseService<Entity, Model, ActiveModel>
where
    Entity: EntityTrait,
    Model: ModelTrait + Send + Sync + 'static,
    ActiveModel: ActiveModelTrait + Send + Sync + 'static,
{
    async fn create(&self, db: &DatabaseConnection, data: ActiveModel) -> Result<Model, AppError> {
        Ok(data.insert(db).await?)
    }

    async fn get_by_id(&self, db: &DatabaseConnection, id: i32) -> Result<Option<Model>, AppError> {
        Ok(Entity::find_by_id(id).one(db).await?)
    }

    async fn get_all(&self, db: &DatabaseConnection) -> Result<Vec<Model>, AppError> {
        Ok(Entity::find().all(db).await?)
    }

    async fn update(
        &self,
        db: &DatabaseConnection,
        id: i32,
        data: ActiveModel,
    ) -> Result<Model, AppError> {
        let model = Entity::find_by_id(id)
            .one(db)
            .await?
            .ok_or(AppError::NotFound)?;
        let mut active_model: ActiveModel = model.into();

        // Update fields individually, handling Optionals correctly:
        // Example:  Assume your Model has a 'name' field.
        if let Some(name) = data.get_name() {
            // Assuming ActiveModel has get_name()
            active_model.set_name(name); // And set_name()
        }
        if let Some(description) = data.get_description() {
            active_model.set_description(description);
        }
        // ... update other fields similarly ...

        Ok(active_model.update(db).await?)
    }

    async fn delete(&self, db: &DatabaseConnection, id: i32) -> Result<(), AppError> {
        let model = Entity::find_by_id(id)
            .one(db)
            .await?
            .ok_or(AppError::NotFound)?;
        Ok(model.delete(db).await?)
    }
}
