use axum::{
    http::StatusCode,
    response::{IntoResponse, Response},
    Json,
};
use sea_orm::error::DbErr;
use serde_json::json;
use std::error::Error as StdError;

#[derive(Debug, thiserror::Error)]
pub enum AppError {
    #[error("Not Found")]
    NotFound,
    #[error("Bad Request: {0}")]
    BadRequest(String),
    #[error("Unauthorized")]
    Unauthorized,
    #[error("Internal Server Error: {0}")]
    InternalServerError(String),
    #[error("Database Error: {0}")]
    DatabaseError(#[from] DbErr),
    #[error("Other Error: {0}")]
    Other(#[from] anyhow::Error), // For other errors using anyhow
}

// Implement the IntoResponse trait for AppError
impl IntoResponse for AppError {
    fn into_response(self) -> Response {
        let (status, message) = match self {
            AppError::NotFound => (StatusCode::NOT_FOUND, "Not Found".to_string()),
            AppError::BadRequest(msg) => (StatusCode::BAD_REQUEST, msg),
            AppError::Unauthorized => (StatusCode::UNAUTHORIZED, "Unauthorized".to_string()),
            AppError::InternalServerError(msg) => (StatusCode::INTERNAL_SERVER_ERROR, msg),
            AppError::DatabaseError(err) => {
                // Log the database error for debugging (important!)
                eprintln!("Database Error: {}", err);
                (
                    StatusCode::INTERNAL_SERVER_ERROR,
                    "Database error occurred".to_string(),
                )
            }
            AppError::Other(err) => {
                eprintln!("Other Error: {}", err);
                (
                    StatusCode::INTERNAL_SERVER_ERROR,
                    "An unexpected error occurred".to_string(),
                )
            }
        };

        let body = Json(json!({
            "error": message,
        }));

        (status, body).into_response()
    }
}

// You can create a From trait implementation if you have a custom error type
// and want to convert it into AppError.  For example, if you had a
// `MyCustomError`:

// impl From<MyCustomError> for AppError {
//     fn from(err: MyCustomError) -> Self {
//        AppError::InternalServerError(err.to_string()) // Or map to a more specific AppError
//     }
// }

// Example usage in your service:
// async fn my_service_function(...) -> Result<..., AppError> {
//     let result = some_fallible_function().map_err(|e| AppError::BadRequest(e.to_string()))?;
//     // ...
// }
